import { Component, OnInit, ViewChild } from '@angular/core';
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcInput from 'wijmo/wijmo.input';
import * as wjcOlap from 'wijmo/wijmo.olap';
import { DataService } from './data.service';
import { FlexGridPager } from './flex-grid-pager';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css']
})
export class AppComponent implements OnInit {

  @ViewChild('theGrid') public theGrid: wjcGrid.FlexGrid;
  @ViewChild('thePivot') public thePivot: wjcOlap.PivotGrid;

  public thePager: FlexGridPager;
  public thePivotPager: FlexGridPager;
  gridDataSource: wjcCore.CollectionView;
  pivotDataSource: wjcOlap.PivotEngine;

  constructor(private dataSvc: DataService) {
    this.gridDataSource = new wjcCore.CollectionView(dataSvc.getData(100));
    this.pivotDataSource = new wjcOlap.PivotEngine({
      itemsSource: dataSvc.getData(100),
      showRowTotals: 'Subtotals',
      rowFields: ['Country', 'Product'],
      valueFields: ['Sales', 'Expenses']
    });
  }
  ngOnInit(): void {
    const groups = ['country', 'product'];
    for (let i = 0; i < groups.length; i++) {
      const prop = groups[i];
      const gd = new wjcCore.PropertyGroupDescription(prop);
      this.gridDataSource.groupDescriptions.push(gd);
    }
    this.thePager = new FlexGridPager(this.theGrid, 8);
    this.thePager.stateChanged.addHandler((s, e) => {
      this.updatePager(this.thePager, 'the-pager-current');
    });
    this.updatePager(this.thePager, 'the-pager-current');

    this.thePivotPager = new FlexGridPager(this.thePivot, 8);
    this.thePivotPager.stateChanged.addHandler((s, e) => {
      this.updatePager(this.thePivotPager, 'the-pivot-pager-current');
    });
    this.updatePager(this.thePivotPager, 'the-pivot-pager-current');
  }

  PagerClick(e: any) {
    const btn = wjcCore.closest(e.target, 'button') as any;
    if (btn) {
      if (btn.id.indexOf('first') > -1) {
        this.thePager.moveToFirstPage();
      } else if (btn.id.indexOf('prev') > -1) {
        this.thePager.moveToPreviousPage();
      } else if (btn.id.indexOf('next') > -1) {
        this.thePager.moveToNextPage();
      } else if (btn.id.indexOf('last') > -1) {
        this.thePager.moveToLastPage();
      }
    }
  }

  PivotPagerClick(e: any) {
    const btn = wjcCore.closest(e.target, 'button') as any;
    if (btn) {
      if (btn.id.indexOf('first') > -1) {
        this.thePivotPager.moveToFirstPage();
      } else if (btn.id.indexOf('prev') > -1) {
        this.thePivotPager.moveToPreviousPage();
      } else if (btn.id.indexOf('next') > -1) {
        this.thePivotPager.moveToNextPage();
      } else if (btn.id.indexOf('last') > -1) {
        this.thePivotPager.moveToLastPage();
      }
    }
  }

  updatePager(pager, current) {
    const caption = 'Page ' + (pager.pageIndex + 1) + ' of ' + pager.pageCount;
    document.getElementById(current).textContent = caption;
  }
}
