
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcOlap from 'wijmo/wijmo.olap';

export class FlexGridPager {

    _pageIndex: number;
    _pageSize: number;
    _pageCount: number;
    _firstRow: number;
    _lastRow: number;
    _g: any;
    _toRefresh: any;

    public stateChanged = new wjcCore.Event();

    constructor(private flex: wjcGrid.FlexGrid, private pagesize: number) {
        const _this = this;
        this._g = wjcCore.asType(flex, wjcGrid.FlexGrid, false);
        this._pageSize = wjcCore.asNumber(this.pagesize, false, true);
        this._pageIndex = 0;
        this.flex.loadedRows.addHandler((s, e) => {
            _this.invalidate();
        });
        this.flex.groupCollapsedChanged.addHandler((s, e) => {
            _this.refresh();
        });
        this.refresh();

        // refresh after collapse/expand on pivot grid
        if (flex instanceof wjcOlap.PivotGrid) {
            flex.hostElement.addEventListener('mousedown', function (e) {
                const icon = wjcCore.closest(e.target, '[wj-pivot-collapse]');
                if (icon) {
                    _this.refresh();
                }
            });
        }
    }

    get itemCount(): number {
        return this._lastRow - this._firstRow + 1;
    }

    get pageCount(): number {
        return this._pageCount;
    }

    get pageIndex(): number {
        return this._pageIndex;
    }
    set pageIndex(value: number) {
        this.moveToPage(value);
    }

    get pageSize(): number {
        return this._pageSize;
    }
    set pageSize(value: number) {
        this._pageSize = wjcCore.asNumber(value);
        this.refresh();
    }

    public moveToFirstPage() {
        return this.moveToPage(0);
    }
    public moveToLastPage() {
        return this.moveToPage(this._pageCount - 1);
    }
    public moveToNextPage() {
        return this.moveToPage(this.pageIndex + 1);
    }
    public moveToPreviousPage() {
        return this.moveToPage(this.pageIndex - 1);
    }
    public moveToPage(index: number) {
        if (index !== this._pageIndex) {
            this._pageIndex = wjcCore.asNumber(index);
            this.refresh();
        }
        return this._pageIndex === index;
    }

    public invalidate() {
        const _this = this;
        if (this._toRefresh) {
            clearTimeout(this._toRefresh);
        }
        this._toRefresh = setTimeout(function () {
            _this._toRefresh = null;
            _this.refresh();
        }, 10);
    }

    public refresh() {
        const _this = this;
        const g = this._g;
        // count non-hidden rows
        const visRows = [];
        for (let i = 0; i < g.rows.length; i++) {
            const r = g.rows[i];
            if (r.visible && !r._getFlag(wjcGrid.RowColFlags.ParentCollapsed)) {
                visRows.push(r);
            }
        }
        // update page count, row indices
        this._pageCount = 1;
        this._firstRow = 0;
        this._lastRow = visRows.length ? visRows.length - 1 : 0;
        if (this._pageSize > 0 && visRows.length) {
            // compute page count and index
            this._pageCount = this._pageSize ? Math.ceil(visRows.length / this._pageSize) : 1;
            this._pageIndex = Math.max(0, Math.min(this._pageCount, this._pageIndex));
            // compute row range for current page
            const start = Math.min(this._pageIndex * this._pageSize, visRows.length - 1);
            this._firstRow = visRows[start].index;
            this._lastRow = visRows[Math.min(start + this._pageSize, visRows.length) - 1].index;
        }
        // show/hide rows
        g.deferUpdate(function () {
            for (let i = 0; i < g.rows.length; i++) {
                g.rows[i].height = (i < _this._firstRow || i > _this._lastRow)
                    ? 0
                    : null;
            }
        });
        // all done
        this.onStateChanged(wjcCore.EventArgs.empty);
    }

    public onStateChanged(e: wjcCore.EventArgs) {
        this.stateChanged.raise(this, e);
    }
}
